@@ -1,8 +1,8 @@
-/*	$OpenBSD: eval.c,v 1.30 2006/04/10 14:38:59 jaredy Exp $	*/
+/*	$OpenBSD: eval.c,v 1.33 2007/08/02 11:05:54 fgsch Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.25.2.1 2007/05/13 19:29:34 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/eval.c,v 1.36 2007/10/25 15:34:29 tg Exp $");
 
 #ifdef MKSH_SMALL
 #define MKSH_NOPWNAM
@@ -98,11 +98,12 @@ char *
 evalstr(const char *cp, int f)
 {
 	XPtrV w;
-	char *dp;
+	char *dp = null;
 
 	XPinit(w, 1);
 	expand(cp, &w, f);
-	dp = (XPsize(w) == 0) ? null : (char*) *XPptrv(w);
+	if (XPsize(w))
+		dp = *XPptrv(w);
 	XPfree(w);
 	return (dp);
 }
@@ -139,8 +140,8 @@ typedef struct SubType {
 	short	stype;		/* [=+-?%#] action after expanded word */
 	short	base;		/* begin position of expanded word */
 	short	f;		/* saved value of f (DOPAT, etc) */
-	struct tbl *var;	/* variable for ${var..} */
 	short	quote;		/* saved value of quote (for ${..[%#]..}) */
+	struct tbl *var;	/* variable for ${var..} */
 	struct SubType *prev;	/* old type */
 	struct SubType *next;	/* poped type (to avoid re-allocating) */
 } SubType;
@@ -278,6 +279,7 @@ expand(const char *cp,	/* input word */
 				if (type < 0) {
 					char *beg, *end, *str;
 
+ unwind_substsyn:
 					sp = varname - 2; /* restore sp */
 					end = (beg = wdcopy(sp, ATEMP)) +
 					    (wdscan(sp, CSUBST) - sp);
@@ -311,6 +313,53 @@ expand(const char *cp,	/* input word */
 					if (stype)
 						sp += slen;
 					switch (stype & 0x7f) {
+					case '0': {
+						char *beg, *mid, *end, *stg;
+						long from = 0, num = -1, flen;
+
+						/* ! DOBLANK,DOBRACE_,DOTILDE */
+						f = DOPAT | (f&DONTRUNCOMMAND) |
+						    DOTEMP_;
+						quote = 0;
+						beg = wdcopy(sp, ATEMP);
+						mid = beg + (wdscan(sp, ADELIM) - sp);
+						stg = beg + (wdscan(sp, CSUBST) - sp);
+						if (mid >= stg)
+							goto unwind_substsyn;
+						mid[-2] = EOS;
+						if (mid[-1] == /*{*/'}') {
+							sp += mid - beg - 1;
+							end = NULL;
+						} else {
+							end = mid +
+							    (wdscan(mid, ADELIM) - mid);
+							if (end >= stg)
+								goto unwind_substsyn;
+							end[-2] = EOS;
+							sp += end - beg - 1;
+						}
+						evaluate(substitute(stg = wdstrip(beg), 0),
+						    &from, KSH_UNWIND_ERROR, true);
+						afree(stg, ATEMP);
+						if (end) {
+							evaluate(stg = wdstrip(mid),
+							    &num, KSH_UNWIND_ERROR, true);
+							afree(stg, ATEMP);
+						}
+						afree(beg, ATEMP);
+						beg = str_val(st->var);
+						flen = strlen(beg);
+						if (from < 0) {
+							if (-from < flen)
+								beg += flen + from;
+						} else
+							beg += from < flen ? from : flen;
+						flen = strlen(beg);
+						if (num < 0 || num > flen)
+							num = flen;
+						x.str = str_nsave(beg, num, ATEMP);
+						goto do_CSUBST;
+					}
 					case '#':
 					case '%':
 						/* ! DOBLANK,DOBRACE_,DOTILDE */
@@ -362,6 +411,7 @@ expand(const char *cp,	/* input word */
 				continue;
 			    }
 			case CSUBST: /* only get here if expanding word */
+ do_CSUBST:
 				sp++; /* ({) skip the } or x */
 				tilde_ok = 0;	/* in case of ${unset:-} */
 				*dp = '\0';
@@ -423,6 +473,13 @@ expand(const char *cp,	/* input word */
 					    "parameter null or not set" :
 					    (debunk(s, s, strlen(s) + 1), s));
 				    }
+				case '0':
+					dp = Xrestpos(ds, dp, st->base);
+					type = XSUB;
+					if (f&DOBLANK)
+						doblank++;
+					st = st->prev;
+					continue;
 				}
 				st = st->prev;
 				type = XBASE;
@@ -712,17 +769,14 @@ varsub(Expand *xp, const char *sp, const char *word,
 		if ((p = cstrchr(sp, '[')) && (p[1] == '*' || p[1] == '@') &&
 		    p[2] == ']') {
 			int n = 0;
-			int max = 0;
 
 			vp = global(arrayname(sp));
 			if (vp->flag & (ISSET|ARRAY))
 				zero_ok = 1;
 			for (; vp; vp = vp->u.array)
-				if (vp->flag & ISSET) {
-					max = vp->index + 1;
+				if (vp->flag & ISSET)
 					n++;
-				}
-			c = n; /* ksh88/ksh93 go for number, not max index */
+			c = n;
 		} else if (c == '*' || c == '@')
 			c = e->loc->argc;
 		else {
@@ -745,7 +799,9 @@ varsub(Expand *xp, const char *sp, const char *word,
 		stype = 0x80;
 		c = word[slen + 0] == CHAR ? word[slen + 1] : 0;
 	}
-	if (ctype(c, C_SUBOP1)) {
+	if (stype == 0x80 && (c == ' ' || c == '0')) {
+		stype |= '0';
+	} else if (ctype(c, C_SUBOP1)) {
 		slen += 2;
 		stype |= c;
 	} else if (ctype(c, C_SUBOP2)) { /* Note: ksh88 allows :%, :%%, etc */
@@ -823,7 +879,7 @@ varsub(Expand *xp, const char *sp, const char *word,
 
 	c = stype&0x7f;
 	/* test the compiler's code generator */
-	if (ctype(c, C_SUBOP2) ||
+	if (ctype(c, C_SUBOP2) || stype == (0x80 | '0') ||
 	    (((stype&0x80) ? *xp->str=='\0' : xp->str==null) ? /* undef? */
 	    c == '=' || c == '-' || c == '?' : c == '+'))
 		state = XBASE;	/* expand word instead of variable value */
@@ -847,6 +903,7 @@ comsub(Expand *xp, const char *cp)
 	s->start = s->str = cp;
 	sold = source;
 	t = compile(s);
+	afree(s, ATEMP);
 	source = sold;
 
 	if (t == NULL)
@@ -1186,9 +1243,7 @@ tilde(char *cp)
 		dp = homedir(cp);
 #endif
 	/* If HOME, PWD or OLDPWD are not set, don't expand ~ */
-	if (dp == null)
-		dp = NULL;
-	return dp;
+	return (dp == null ? NULL : dp);
 }
 
 #ifndef MKSH_NOPWNAM
