@@ -1,8 +1,8 @@
-/*	$OpenBSD: exec.c,v 1.46 2006/04/10 14:38:59 jaredy Exp $	*/
+/*	$OpenBSD: exec.c,v 1.48 2007/09/05 19:02:01 otto Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.29.2.1 2007/05/13 19:29:34 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/exec.c,v 1.39 2007/10/25 13:27:00 tg Exp $");
 
 static int comexec(struct op *, struct tbl *volatile, const char **,
     int volatile);
@@ -35,14 +35,6 @@ execute(struct op *volatile t,
 	if (t == NULL)
 		return 0;
 
-	/* Is this the end of a pipeline?  If so, we want to evaluate the
-	 * command arguments
-	bool eval_done = false;
-	if ((flags&XFORK) && !(flags&XEXEC) && (flags&XPCLOSE)) {
-		eval_done = true;
-		tp = eval_execute_args(t, &ap);
-	}
-	 */
 	if ((flags&XFORK) && !(flags&XEXEC) && t->type != TPIPE)
 		return exchild(t, flags & ~XTIME, -1); /* run in sub-process */
 
@@ -70,8 +62,8 @@ execute(struct op *volatile t,
 			shf_fprintf(shl_out, "%s",
 				substitute(str_val(global("PS4")), 0));
 			for (i = 0; ap[i]; i++)
-				shf_fprintf(shl_out, "%s%s", ap[i],
-				    ap[i + 1] ? space : newline);
+				shf_fprintf(shl_out, "%s%c", ap[i],
+				    ap[i + 1] ? ' ' : '\n');
 			shf_flush(shl_out);
 		}
 		if (ap[0])
@@ -95,7 +87,7 @@ execute(struct op *volatile t,
 				 */
 				if (tp && tp->type == CSHELL &&
 				    (tp->flag & SPEC_BI))
-					errorf(null);
+					errorfz();
 				/* Deal with FERREXIT, quitenv(), etc. */
 				goto Break;
 			}
@@ -348,11 +340,12 @@ execute(struct op *volatile t,
 
 			cargs.ro = t->args;
 			execve(t->str, cargs.rw, up);
+			rv = errno;
 		}
-		if (errno == ENOEXEC)
+		if (rv == ENOEXEC)
 			scriptexec(t, (const char **)up);
 		else
-			errorf("%s: %s", s, strerror(errno));
+			errorf("%s: %s", s, strerror(rv));
 	}
  Break:
 	exstat = rv;
@@ -361,9 +354,9 @@ execute(struct op *volatile t,
 	if ((flags&XEXEC))
 		unwind(LEXIT);	/* exit child */
 	if (rv != 0 && !(flags & XERROK)) {
+		trapsig(SIGERR_);
 		if (Flag(FERREXIT))
 			unwind(LERROR);
-		trapsig(SIGERR_);
 	}
 	return rv;
 }
@@ -405,9 +398,9 @@ comexec(struct op *t, struct tbl *volatile tp, const char **ap,
 	 * before we can do a path search (in case the assignments change
 	 * PATH).
 	 * Odd cases:
-	 *   FOO=bar exec > /dev/null		FOO is kept but not exported
+	 *   FOO=bar exec >/dev/null		FOO is kept but not exported
 	 *   FOO=bar exec foobar		FOO is exported
-	 *   FOO=bar command exec > /dev/null	FOO is neither kept nor exported
+	 *   FOO=bar command exec >/dev/null	FOO is neither kept nor exported
 	 *   FOO=bar command			FOO is neither kept nor exported
 	 *   PATH=... foobar			use new PATH in foobar search
 	 */
@@ -485,8 +478,8 @@ comexec(struct op *t, struct tbl *volatile tp, const char **ap,
 			if (i == 0)
 				shf_fprintf(shl_out, "%s",
 					substitute(str_val(global("PS4")), 0));
-			shf_fprintf(shl_out, "%s%s", cp,
-			    t->vars[i + 1] ? space : newline);
+			shf_fprintf(shl_out, "%s%c", cp,
+			    t->vars[i + 1] ? ' ' : '\n');
 			if (!t->vars[i + 1])
 				shf_flush(shl_out);
 		}
@@ -514,7 +507,7 @@ comexec(struct op *t, struct tbl *volatile tp, const char **ap,
 
 	case CFUNC:			/* function call */
 	    {
-		volatile int old_xflag;
+		volatile char old_xflag;
 		volatile Tflag old_inuse;
 		const char *volatile old_kshname;
 
@@ -537,9 +530,10 @@ comexec(struct op *t, struct tbl *volatile tp, const char **ap,
 				break;
 			}
 			if (include(tp->u.fpath, 0, NULL, 0) < 0) {
+				rv = errno;
 				warningf(true,
 				    "%s: can't open function definition file %s - %s",
-				    cp, tp->u.fpath, strerror(errno));
+				    cp, tp->u.fpath, strerror(rv));
 				rv = 127;
 				break;
 			}
@@ -576,7 +570,7 @@ comexec(struct op *t, struct tbl *volatile tp, const char **ap,
 		}
 
 		old_xflag = Flag(FXTRACE);
-		Flag(FXTRACE) = tp->flag & TRACE ? true : false;
+		Flag(FXTRACE) = tp->flag & TRACE ? 1 : 0;
 
 		old_inuse = tp->flag & FINUSE;
 		tp->flag |= FINUSE;
@@ -735,8 +729,20 @@ scriptexec(struct op *tp, const char **ap)
 			if (*cp)
 				*tp->args-- = (char *)cp;
 		}
-	}
  noshebang:
+		fd = buf[0] << 8 | buf[1];
+		if ((fd == /* OMAGIC */ 0407) ||
+		    (fd == /* NMAGIC */ 0410) ||
+		    (fd == /* ZMAGIC */ 0413) ||
+		    (fd == /* QMAGIC */ 0314) ||
+		    (fd == /* ECOFF_I386 */ 0x4C01) ||
+		    (fd == /* ECOFF_M68K */ 0x0150 || fd == 0x5001) ||
+		    (fd == /* ECOFF_SH */   0x0500 || fd == 0x0005) ||
+		    (fd == 0x7F45 && buf[2] == 'L' && buf[3] == 'F') ||
+		    (fd == /* “MZ” */ 0x4D5A) ||
+		    (fd == /* gzip */ 0x1F8B))
+			errorf("%s: not executable: magic %04X", tp->str, fd);
+	}
 #endif
 	args.ro = tp->args;
 	*args.ro = sh;
@@ -867,7 +873,7 @@ findcom(const char *name, int flags)
 	struct tbl *tp = NULL, *tbi;
 	int insert = Flag(FTRACKALL);	/* insert if not found */
 	char *fpath;			/* for function autoloading */
-	const char *npath;
+	union mksh_cchack npath;
 
 	if (vstrchr(name, '/')) {
 		insert = 0;
@@ -920,14 +926,21 @@ findcom(const char *name, int flags)
 			}
 			tp->flag = DEFINED;	/* make ~ISSET */
 		}
-		npath = search(name, flags & FC_DEFPATH ? def_path : path,
+		npath.ro = search(name, flags & FC_DEFPATH ? def_path : path,
 		    X_OK, &tp->u2.errno_);
-		if (npath) {
-			tp->val.s = str_save(npath, APERM);
+		if (npath.ro) {
+			/* XXX unsure about this, oksh does that
+			if (tp == &temp)
+				tp->val.s = npath.ro;
+			else */ {
+				tp->val.s = str_save(npath.ro, APERM);
+				if (npath.ro != name)
+					afree(npath.rw, ATEMP);
+			}
 			tp->flag |= ISSET|ALLOC;
 		} else if ((flags & FC_FUNC) &&
 		    (fpath = str_val(global("FPATH"))) != null &&
-		    (npath = search(name, fpath, R_OK,
+		    (npath.ro = search(name, fpath, R_OK,
 		    &tp->u2.errno_)) != NULL) {
 			/* An undocumented feature of at&t ksh is that it
 			 * searches FPATH if a command is not found, even
@@ -937,7 +950,7 @@ findcom(const char *name, int flags)
 			tp = &temp;
 			tp->type = CFUNC;
 			tp->flag = DEFINED; /* make ~ISSET */
-			tp->u.fpath = npath;
+			tp->u.fpath = npath.ro;
 		}
 	}
 	return tp;
@@ -1137,11 +1150,13 @@ iosetup(struct ioword *iop, struct tbl *tp)
 	}
 	if (u < 0) {
 		/* herein() may already have printed message */
-		if (u == -1)
+		if (u == -1) {
+			u = errno;
 			warningf(true, "cannot %s %s: %s",
 			    iotype == IODUP ? "dup" :
 			    (iotype == IOREAD || iotype == IOHERE) ?
-			    "open" : "create", cp, strerror(errno));
+			    "open" : "create", cp, strerror(u));
+		}
 		return -1;
 	}
 	/* Do not save if it has already been redirected (i.e. "cat >x >y"). */
@@ -1163,10 +1178,13 @@ iosetup(struct ioword *iop, struct tbl *tp)
 		close(iop->unit);
 	else if (u != iop->unit) {
 		if (ksh_dup2(u, iop->unit, true) < 0) {
+			int ev;
+
+			ev = errno;
 			warningf(true,
 			    "could not finish (dup) redirection %s: %s",
 			    snptreef(NULL, 32, "%R", &iotmp),
-			    strerror(errno));
+			    strerror(ev));
 			if (iotype != IODUP)
 				close(u);
 			return -1;
@@ -1212,9 +1230,10 @@ herein(const char *content, int sub)
 	 */
 	h = maketemp(ATEMP, TT_HEREDOC_EXP, &e->temps);
 	if (!(shf = h->shf) || (fd = open(h->name, O_RDONLY, 0)) < 0) {
+		fd = errno;
 		warningf(true, "can't %s temporary file %s: %s",
 		    !shf ? "create" : "open",
-		    h->name, strerror(errno));
+		    h->name, strerror(fd));
 		if (shf)
 			shf_close(shf);
 		return -2 /* special to iosetup(): don't print error */;
@@ -1244,9 +1263,11 @@ herein(const char *content, int sub)
 	quitenv(NULL);
 
 	if (shf_close(shf) == EOF) {
+		i = errno;
 		close(fd);
-		warningf(true, "error writing %s: %s", h->name,
-		    strerror(errno));
+		fd = errno;
+		warningf(true, "error writing %s: %s, %s", h->name,
+		    strerror(i), strerror(fd));
 		return -2; /* special to iosetup(): don't print error */
 	}
 
@@ -1381,9 +1402,6 @@ pr_list(char *const *ap)
  *	[[ ... ]] evaluation routines
  */
 
-extern const char *const dbtest_tokens[];
-extern const char db_close[];
-
 /* Test if the current token is a whatever.  Accepts the current token if
  * it is.  Returns 0 if it is not, non-zero if it is (in the case of
  * TM_UNOP and TM_BINOP, the returned value is a Test_op).
