@@ -3,10 +3,10 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.45.2.1 2007/05/13 19:29:36 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/histrap.c,v 1.57 2007/10/25 15:34:30 tg Exp $");
 
 Trap sigtraps[NSIG + 1];
-static struct sigaction Sigact_ign, Sigact_trap;
+static struct sigaction Sigact_ign;
 
 #if HAVE_PERSISTENT_HISTORY
 static int hist_count_lines(unsigned char *, int);
@@ -38,7 +38,7 @@ int
 c_fc(const char **wp)
 {
 	struct shf *shf;
-	struct temp *tf = NULL;
+	struct temp *tf;
 	const char *p;
 	char *editor = NULL;
 	int gflag = 0, lflag = 0, nflag = 0, sflag = 0, rflag = 0;
@@ -173,12 +173,13 @@ c_fc(const char **wp)
 	/* List history */
 	if (lflag) {
 		char *s, *t;
-		const char *nfmt = nflag ? "\t" : "%d\t";
 
 		for (hp = rflag ? hlast : hfirst;
 		    hp >= hfirst && hp <= hlast; hp += rflag ? -1 : 1) {
-			shf_fprintf(shl_stdout, nfmt,
-			    hist_source->line - (int) (histptr - hp));
+			if (!nflag)
+				shf_fprintf(shl_stdout, "%d",
+				    hist_source->line - (int)(histptr - hp));
+			shf_putc('\t', shl_stdout);
 			/* print multi-line commands correctly */
 			for (s = *hp; (t = strchr(s, '\n')); s = t)
 				shf_fprintf(shl_stdout, "%.*s\t",
@@ -360,11 +361,9 @@ hist_get(const char *str, int approx, int allow_cur)
 		int anchored = *str == '?' ? (++str, 0) : 1;
 
 		/* the -1 is to avoid the current fc command */
-		n = findhist(histptr - history - 1, 0, str, anchored);
-		if (n < 0) {
+		if ((n = findhist(histptr - history - 1, 0, str, anchored)) < 0)
 			bi_errorf("%s: not in history", str);
-			hp = NULL;
-		} else
+		else
 			hp = &history[n];
 	}
 	return hp;
@@ -378,9 +377,7 @@ hist_get_newest(int allow_cur)
 		bi_errorf("no history (yet)");
 		return NULL;
 	}
-	if (allow_cur)
-		return histptr;
-	return histptr - 1;
+	return (allow_cur ? histptr : histptr - 1);
 }
 
 /* Return a pointer to the newest command in the history */
@@ -422,7 +419,7 @@ histpos(void)
 int
 histnum(int n)
 {
-	int	last = histptr - history;
+	int last = histptr - history;
 
 	if (n < 0 || n >= last) {
 		current = histptr;
@@ -658,8 +655,8 @@ hist_init(Source *s)
 		/*
 		 * we have some data
 		 */
-		base = (unsigned char *)mmap(0, hsize, PROT_READ,
-		    MAP_FILE|MAP_PRIVATE, histfd, 0);
+		base = mmap(NULL, hsize, PROT_READ, MAP_FILE | MAP_PRIVATE,
+		    histfd, 0);
 		/*
 		 * check on its validity
 		 */
@@ -750,15 +747,12 @@ hist_shrink(unsigned char *oldbase, int oldbytes)
 	shf_snprintf(nfile, sizeof(nfile), "%s.%d", hname, (int)procpid);
 	if ((fd = open(nfile, O_CREAT | O_TRUNC | O_WRONLY, 0600)) < 0)
 		return 1;
-	if (fstat(histfd, &statb) >= 0)
-		chown(nfile, statb.st_uid, statb.st_gid);
+	if (fstat(histfd, &statb) >= 0 &&
+	    chown(nfile, statb.st_uid, statb.st_gid))
+		goto errout;
 
-	if (sprinkle(fd)) {
-		close(fd);
-		unlink(nfile);
-		return 1;
-	}
-	if (write(fd, nbase, nbytes) != nbytes) {
+	if (sprinkle(fd) || write(fd, nbase, nbytes) != nbytes) {
+ errout:
 		close(fd);
 		unlink(nfile);
 		return 1;
@@ -890,8 +884,8 @@ writehistfile(int lno, char *cmd)
 		if (sizenow > hsize) {
 			/* someone has added some lines */
 			bytes = sizenow - hsize;
-			base = (unsigned char *)mmap(0, sizenow,
-			    PROT_READ, MAP_FILE|MAP_PRIVATE, histfd, 0);
+			base = mmap(NULL, sizenow, PROT_READ,
+			    MAP_FILE | MAP_PRIVATE, histfd, 0);
 			if (base == MAP_FAILED)
 				goto bad;
 			new = base + hsize;
@@ -920,8 +914,10 @@ writehistfile(int lno, char *cmd)
 	hdr[2] = (lno>>16)&0xff;
 	hdr[3] = (lno>>8)&0xff;
 	hdr[4] = lno&0xff;
-	(void) write(histfd, hdr, 5);
-	(void) write(histfd, cmd, strlen(cmd)+1);
+	bytes = strlen(cmd) + 1;
+	if ((write(histfd, hdr, 5) != 5) ||
+	    (write(histfd, cmd, bytes) != bytes))
+		goto bad;
 	hsize = lseek(histfd, 0L, SEEK_END);
 	(void) flock(histfd, LOCK_UN);
 	return;
@@ -949,11 +945,14 @@ sprinkle(int fd)
 }
 #endif
 
-#if HAVE_SYS_SIGNAME
-#elif HAVE__SYS_SIGNAME
-#define sys_signame	_sys_signame
-#else
-#include "signames.c"
+#if !HAVE_SYS_SIGNAME
+static const struct mksh_sigpair {
+	int nr;
+	const char *const name;
+} mksh_sigpairs[] = {
+#include "signames.inc"
+	{ 0, NULL }
+};
 #endif
 
 void
@@ -969,7 +968,7 @@ inittraps(void)
 			sigtraps[i].name = "ERR";
 			sigtraps[i].mess = "Error handler";
 		} else {
-#if HAVE_SYS_SIGNAME || HAVE__SYS_SIGNAME
+#if HAVE_SYS_SIGNAME
 			cs = sys_signame[i];
 #else
 			const struct mksh_sigpair *pair = mksh_sigpairs;
@@ -1006,8 +1005,6 @@ inittraps(void)
 	sigemptyset(&Sigact_ign.sa_mask);
 	Sigact_ign.sa_flags = 0; /* interruptible */
 	Sigact_ign.sa_handler = SIG_IGN;
-	Sigact_trap = Sigact_ign;
-	Sigact_trap.sa_handler = trapsig;
 
 	sigtraps[SIGINT].flags |= TF_DFL_INTR | TF_TTY_INTR;
 	sigtraps[SIGQUIT].flags |= TF_DFL_INTR | TF_TTY_INTR;
@@ -1055,14 +1052,21 @@ gettrap(const char *name, int igncase)
 {
 	int n = NSIG + 1;
 	Trap *p;
+	const char *n2;
+	int (*cmpfunc)(const char *, const char *) = strcmp;
 
 	if (ksh_isdigit(*name)) {
 		if (getn(name, &n) && 0 <= n && n < NSIG)
 			return (&sigtraps[n]);
-	} else for (p = sigtraps; --n >= 0; p++)
-		if (!(igncase ? strcasecmp : strcmp)(p->name, name) ||
-		   (!strncasecmp(name, "SIG", 3) &&
-		    !(igncase ? strcasecmp : strcmp)(p->name, name + 3)))
+		else
+			return (NULL);
+	}
+
+	n2 = strncasecmp(name, "SIG", 3) ? NULL : name + 3;
+	if (igncase)
+		cmpfunc = strcasecmp;
+	for (p = sigtraps; --n >= 0; p++)
+		if (!cmpfunc(p->name, name) || (n2 && !cmpfunc(p->name, n2)))
 			return (p);
 	return (NULL);
 }
@@ -1336,7 +1340,7 @@ setsig(Trap *p, sig_t f, int flags)
 	 * all users of shtrap are lifetime users (SIGCHLD, SIGALRM, SIGWINCH).
 	 */
 	if (!(flags & SS_USER))
-		p->shtrap = NULL;
+		p->shtrap = (sig_t)NULL;
 	if (flags & SS_SHTRAP) {
 		p->shtrap = f;
 		f = trapsig;
