@@ -1,8 +1,8 @@
-/*	$OpenBSD: jobs.c,v 1.35 2006/02/06 16:47:07 jmc Exp $	*/
+/*	$OpenBSD: jobs.c,v 1.36 2007/09/06 19:57:47 otto Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.21.2.1 2007/05/13 19:29:36 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/jobs.c,v 1.32 2007/10/25 15:23:09 tg Exp $");
 
 /* Order important! */
 #define PRUNNING	0
@@ -49,7 +49,7 @@ struct job {
 	Job *next;		/* next job in list */
 	int job;		/* job number: %n */
 	int flags;		/* see JF_* */
-	int state;		/* job state */
+	volatile int state;	/* job state */
 	int status;		/* exit status of last process */
 	pid_t pgrp;		/* process group of job */
 	pid_t ppid;		/* pid of process that forked job */
@@ -214,11 +214,14 @@ j_change(void)
 	int i;
 
 	if (Flag(FMONITOR)) {
+		bool use_tty = Flag(FTALKING);
+
 		/* Don't call tcgetattr() 'til we own the tty process group */
-		tty_init(false);
+		if (use_tty)
+			tty_init(false);
 
 		/* no controlling tty, no SIGT* */
-		ttypgrp_ok = tty_fd >= 0 && tty_devtty;
+		ttypgrp_ok = use_tty && tty_fd >= 0 && tty_devtty;
 
 		if (ttypgrp_ok && (our_pgrp = getpgrp()) < 0) {
 			warningf(false, "j_init: getpgrp() failed: %s",
@@ -264,7 +267,7 @@ j_change(void)
 				our_pgrp = kshpid;
 			}
 		}
-		if (!ttypgrp_ok)
+		if (use_tty && !ttypgrp_ok)
 			warningf(false, "warning: won't have full job control");
 		if (tty_fd >= 0)
 			tcgetattr(tty_fd, &tty_state);
@@ -290,8 +293,7 @@ j_change(void)
 
 /* execute tree in child subprocess */
 int
-exchild(struct op *t, int flags,
-    int close_fd)	/* used if XPCLOSE or XCCLOSE */
+exchild(struct op *t, int flags, /* used if XPCLOSE or XCCLOSE */ int close_fd)
 {
 	static Proc	*last_proc;	/* for pipelines */
 
@@ -369,6 +371,9 @@ exchild(struct op *t, int flags,
 	else
 		p->pid = i;
 
+	/* Ensure next child gets a (slightly) different $RANDOM sequence */
+	change_random(((unsigned long)p->pid << 1) | (ischild ? 1 : 0));
+
 	/* job control set up */
 	if (Flag(FMONITOR) && !(flags&XXCOM)) {
 		int	dotty = 0;
@@ -405,7 +410,7 @@ exchild(struct op *t, int flags,
 				    SS_RESTORE_DFL|SS_FORCE);
 		}
 		if (Flag(FBGNICE) && (flags & XBGND))
-			nice(4);
+			i = nice(4);
 		if ((flags & XBGND) && !Flag(FMONITOR)) {
 			setsig(&sigtraps[SIGINT], SIG_IGN,
 			    SS_RESTORE_IGN|SS_FORCE);
@@ -427,8 +432,10 @@ exchild(struct op *t, int flags,
 		tty_close();
 		cleartraps();
 		execute(t, (flags & XERROK) | XEXEC); /* no return */
-		internal_warningf("exchild: execute() returned");
 #ifndef MKSH_SMALL
+		if (t->type == TPIPE)
+			unwind(LLEAVE);
+		internal_warningf("exchild: execute() returned");
 		fptreef(shl_out, 2, "exchild: tried to execute {\n%T\n}\n", t);
 		shf_flush(shl_out);
 #endif
@@ -437,8 +444,6 @@ exchild(struct op *t, int flags,
 	}
 
 	/* shell (parent) stuff */
-	/* Ensure next child gets a (slightly) different $RANDOM sequence */
-	change_random();
 	if (!(flags & XPIPEO)) {	/* last process in a job */
 		j_startjob(j);
 		if (flags & XCOPROC) {
@@ -632,7 +637,7 @@ j_resume(const char *cp, int bg)
 		}
 		shprintf("%s%s", p->command, p->next ? "| " : null);
 	}
-	shprintf(newline);
+	shf_putc('\n', shl_stdout);
 	shf_flush(shl_stdout);
 	if (running)
 		j->state = PRUNNING;
@@ -649,6 +654,7 @@ j_resume(const char *cp, int bg)
 			if (ttypgrp_ok &&
 			    tcsetpgrp(tty_fd, (j->flags & JF_SAVEDTTYPGRP) ?
 			    j->saved_ttypgrp : j->pgrp) < 0) {
+				rv = errno;
 				if (j->flags & JF_SAVEDTTY)
 					tcsetattr(tty_fd, TCSADRAIN, &tty_state);
 				sigprocmask(SIG_SETMASK, &omask,
@@ -657,7 +663,7 @@ j_resume(const char *cp, int bg)
 				    tty_fd,
 				    (int) ((j->flags & JF_SAVEDTTYPGRP) ?
 				    j->saved_ttypgrp : j->pgrp),
-				    strerror(errno));
+				    strerror(rv));
 				return 1;
 			}
 		}
@@ -668,7 +674,7 @@ j_resume(const char *cp, int bg)
 	}
 
 	if (j->state == PRUNNING && killpg(j->pgrp, SIGCONT) < 0) {
-		int	err = errno;
+		int err = errno;
 
 		if (!bg) {
 			j->flags &= ~JF_FG;
@@ -905,8 +911,8 @@ j_waitj(Job *j,
 	if (!Flag(FMONITOR))
 		flags |= JW_STOPPEDWAIT;
 
-	while ((volatile int) j->state == PRUNNING ||
-	    ((flags & JW_STOPPEDWAIT) && (volatile int) j->state == PSTOPPED)) {
+	while (j->state == PRUNNING ||
+	    ((flags & JW_STOPPEDWAIT) && j->state == PSTOPPED)) {
 		sigsuspend(&sm_default);
 		if (fatal_trap) {
 			int oldf = j->flags & (JF_WAITING|JF_W_ASYNCNOTIFY);
@@ -1282,7 +1288,7 @@ j_print(Job *j, int how, struct shf *shf)
 		while (p && p->state == state && p->status == status) {
 			if (how == JP_LONG)
 				shf_fprintf(shf, "%s%5d %-20s %s%s", filler,
-				    (int)p->pid, space, p->command,
+				    (int)p->pid, " ", p->command,
 				    p->next ? "|" : null);
 			else if (how == JP_MEDIUM)
 				shf_fprintf(shf, " %s%s", p->command,
@@ -1291,7 +1297,7 @@ j_print(Job *j, int how, struct shf *shf)
 		}
 	}
 	if (output)
-		shf_fprintf(shf, newline);
+		shf_putc('\n', shf);
 }
 
 /* Convert % sequence to job
