@@ -1,8 +1,8 @@
-/*	$OpenBSD: lex.c,v 1.42 2006/07/10 17:12:41 beck Exp $	*/
+/*	$OpenBSD: lex.c,v 1.43 2007/06/02 16:40:59 moritz Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.29.2.1 2007/05/13 19:29:37 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/lex.c,v 1.48 2007/10/25 15:27:54 tg Exp $");
 
 /* Structure to keep track of the lexing state and the various pieces of info
  * needed for each particular state. */
@@ -36,6 +36,24 @@ struct lex_state {
 #define ls_sbquote ls_info.u_sbquote
 		} u_sbquote;
 
+		/* =(...) */
+		struct sletarray_info {
+			int nparen;	/* count open parentheses */
+#define ls_sletarray ls_info.u_sletarray
+		} u_sletarray;
+
+		/* ADELIM */
+		struct sadelim_info {
+			unsigned char nparen;	/* count open parentheses */
+#define SADELIM_BASH	0
+#define SADELIM_MAKE	1
+			unsigned char style;
+			unsigned char delimiter;
+			unsigned char num;
+			unsigned char flags;	/* ofs. into sadelim_flags[] */
+#define ls_sadelim ls_info.u_sadelim
+		} u_sadelim;
+
 		Lex_state *base;	/* used to point to next state block */
 	} ls_info;
 };
@@ -70,7 +88,11 @@ static int ignore_backslash_newline;
 #define	getsc_()	((*source->str != '\0') && !(source->flags & SF_FIRST) \
 			 ? *source->str++ : getsc__())
 
+#ifdef MKSH_SMALL
 #define STATE_BSIZE	32
+#else
+#define STATE_BSIZE	48
+#endif
 
 #define PUSH_STATE(s)	do { \
 			    if (++statep == state_info.end) \
@@ -121,6 +143,9 @@ yylex(int cf)
 		*wp++ = OQUOTE;	 /* enclose arguments in (double) quotes */
 		state = SLETPAREN;
 		statep->ls_sletparen.nparen = 0;
+	} else if (cf&LETARRAY) {
+		state = SLETARRAY;
+		statep->ls_sletarray.nparen = 0;
 	} else {		/* normal lexing */
 		state = (cf & HEREDELIM) ? SHEREDELIM : SBASE;
 		while ((c = getsc()) == ' ' || c == '\t')
@@ -146,6 +171,51 @@ yylex(int cf)
 	    ((state == SBASE || state == SHEREDELIM) && ctype(c, C_LEX1)))) {
 		Xcheck(ws, wp);
 		switch (state) {
+		case SADELIM:
+			if (c == '(')
+				statep->ls_sadelim.nparen++;
+			else if (c == ')')
+				statep->ls_sadelim.nparen--;
+			else if (statep->ls_sadelim.nparen == 0 &&
+			    (c == /*{*/ '}' || c == statep->ls_sadelim.delimiter)) {
+#ifdef notyet
+				if (statep->ls_sadelim.style == SADELIM_MAKE &&
+				    statep->ls_sadelim.num == 1) {
+					if (c == /*{*/'}')
+						yyerror("%s: expected '%c' %s\n",
+						    T_synerr,
+						    statep->ls_sadelim.delimiter,
+					/*{*/	    "before '}'");
+					else {
+						*wp++ = ADELIM;
+						*wp++ = c;	/* .delimiter */
+						while ((c = getsc()) != /*{*/ '}') {
+							if (!c) {
+								yyerror("%s: expected '%c' %s\n",
+								    T_synerr,
+							/*{*/	    '}', "at end of input");
+							} else if (strchr(sadelim_flags[statep->ls_sadelim.flags], c)) {
+								*wp++ = CHAR;
+								*wp++ = c;
+							} else {
+								char Ttmp[15] = "instead of ' '";
+
+								Ttmp[12] = c;
+								yyerror("%s: expected '%c' %s\n",
+								    T_synerr,
+							/*{*/	    '}', Ttmp);
+							}
+						}
+					}
+				}
+#endif /* SADELIM_MAKE */
+				*wp++ = ADELIM;
+				*wp++ = c;
+				if (c == /*{*/ '}' || --statep->ls_sadelim.num == 0)
+					POP_STATE();
+				break;
+			}
+			/* FALLTHROUGH */
 		case SBASE:
 			if (c == '[' && (cf & (VARASN|ARRAYVAR))) {
 				*wp = EOS; /* temporary */
@@ -220,7 +290,7 @@ yylex(int cf)
 				case '"':
 					if ((cf & HEREDOC))
 						goto heredocquote;
-					/* FALLTROUGH */
+					/* FALLTHROUGH */
 				case '\\':
 				case '$': case '`':
 					*wp++ = QCHAR, *wp++ = c;
@@ -261,6 +331,33 @@ yylex(int cf)
 					if (c == ':') {
 						*wp++ = CHAR, *wp++ = c;
 						c = getsc();
+						if (c == ':') {
+							*wp++ = CHAR;
+							*wp++ = '0';
+							*wp++ = ADELIM;
+							*wp++ = ':';
+							PUSH_STATE(SADELIM);
+							statep->ls_sadelim.style = SADELIM_BASH;
+							statep->ls_sadelim.delimiter = ':';
+							statep->ls_sadelim.num = 1;
+							statep->ls_sadelim.nparen = 0;
+							break;
+						} else if (ksh_isdigit(c) ||
+						    c == '('/*)*/ || c == ' ' ||
+						    c == '$' /* XXX what else? */) {
+							/* substring subst. */
+							if (c != ' ') {
+								*wp++ = CHAR;
+								*wp++ = ' ';
+							}
+							ungetsc(c);
+							PUSH_STATE(SADELIM);
+							statep->ls_sadelim.style = SADELIM_BASH;
+							statep->ls_sadelim.delimiter = ':';
+							statep->ls_sadelim.num = 2;
+							statep->ls_sadelim.nparen = 0;
+							break;
+						}
 					}
 					/* If this is a trim operation,
 					 * treat (,|,) specially in STBRACE.
@@ -535,6 +632,17 @@ yylex(int cf)
 				++statep->ls_sletparen.nparen;
 			goto Sbase2;
 
+		case SLETARRAY:	/* LETARRAY: =( ... ) */
+			if (c == '('/*)*/)
+				++statep->ls_sletarray.nparen;
+			else if (c == /*(*/')')
+				if (statep->ls_sletarray.nparen-- == 0) {
+					c = 0;
+					goto Done;
+				}
+			*wp++ = CHAR, *wp++ = c;
+			break;
+
 		case SHEREDELIM:	/* <<,<<- delimiter */
 			/* XXX chuck this state (and the next) - use
 			 * the existing states ($ and \`..` should be
@@ -607,6 +715,9 @@ yylex(int cf)
 		/* XXX figure out what is missing */
 		yyerror("no closing quote\n");
 
+	if (state == SLETARRAY && statep->ls_sletarray.nparen != -1)
+		yyerror("%s: ')' missing\n", T_synerr);
+
 	/* This done to avoid tests for SHEREDELIM wherever SBASE tested */
 	if (state == SHEREDELIM)
 		state = SBASE;
@@ -664,8 +775,7 @@ yylex(int cf)
 				     * ((...); (...))
 				     * and similar is broken
 				     */
-				    (c == '(' /*)*/ ) ? MDPAREN :
-				    YYERRCODE;
+				    /* c == '(' ) */ MDPAREN;
 			else if (c == '|' && c2 == '&')
 				c = COPROC;
 			else
@@ -680,7 +790,8 @@ yylex(int cf)
 
 	*wp++ = EOS;		/* terminate word */
 	yylval.cp = Xclose(ws, wp);
-	if (state == SWORD || state == SLETPAREN)	/* ONEWORD? */
+	if (state == SWORD || state == SLETPAREN ||
+	    state == SLETARRAY)	/* ONEWORD? */
 		return LWORD;
 	ungetsc(c);		/* unget terminator */
 
@@ -809,7 +920,7 @@ yyerror(const char *fmt, ...)
 	va_start(va, fmt);
 	shf_vfprintf(shl_out, fmt, va);
 	va_end(va);
-	errorf(null);
+	errorfz();
 }
 
 /*
@@ -871,10 +982,10 @@ getsc__(void)
 
 		case SWORDSEP:
 			if (*s->u.strv == NULL) {
-				s->start = s->str = newline;
+				s->start = s->str = "\n";
 				s->type = SEOF;
 			} else {
-				s->start = s->str = space;
+				s->start = s->str = " ";
 				s->type = SWORDS;
 			}
 			break;
@@ -969,7 +1080,11 @@ getsc_line(Source *s)
 		ksh_tmout_state = TMOUT_READING;
 		alarm(ksh_tmout);
 	}
-	if (have_tty && (Flag(FVI) || Flag(FEMACS) || Flag(FGMACS))) {
+	if (have_tty && (
+#ifndef MKSH_NOVI
+	    Flag(FVI) ||
+#endif
+	    Flag(FEMACS) || Flag(FGMACS))) {
 		int nread;
 
 		nread = x_read(xp, LINE);
