@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.41 2006/04/10 14:38:59 jaredy Exp $	*/
+/*	$OpenBSD: main.c,v 1.43 2007/05/31 20:47:44 otto Exp $	*/
 /*	$OpenBSD: tty.c,v 1.9 2006/03/14 22:08:01 deraadt Exp $	*/
 /*	$OpenBSD: io.c,v 1.22 2006/03/17 16:30:13 millert Exp $	*/
 /*	$OpenBSD: table.c,v 1.12 2005/12/11 20:31:21 otto Exp $	*/
@@ -13,7 +13,7 @@
 #include <locale.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/main.c,v 1.74.2.1 2007/05/13 19:29:37 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/main.c,v 1.89 2007/10/25 15:23:10 tg Exp $");
 
 extern char **environ;
 
@@ -34,8 +34,12 @@ static const char *initcoms[] = {
 	"typeset", "-x", "SHELL", "PATH", "HOME", NULL,
 	"typeset", "-i", "PPID", "OPTIND=1", NULL,
 	"eval", "typeset -i RANDOM SECONDS=\"${SECONDS-0}\" TMOUT=\"${TMOUT-0}\"", NULL,
+	"alias", "integer=typeset -i", "local=typeset", NULL,
+	NULL
+};
+
+static const char *initcoms_korn[] = {
 	"alias",
-	 /* Standard ksh aliases */
 	"hash=alias -t",	/* not "alias -t --": hash -r needs to work */
 	"type=whence -v",
 	"stop=kill -STOP",
@@ -43,11 +47,9 @@ static const char *initcoms[] = {
 	"autoload=typeset -fu",
 	"functions=typeset -f",
 	"history=fc -l",
-	"integer=typeset -i",
 	"nohup=nohup ",
-	"local=typeset",
 	"r=fc -e -",
-	 /* Aliases that are builtin commands in at&t */
+	"source=PATH=$PATH:. command .",
 	"login=exec login",
 	NULL,
 	 /* this is what at&t ksh seems to track, with the addition of emacs */
@@ -62,12 +64,11 @@ static int initio_done;
 int
 main(int argc, const char *argv[])
 {
-	int i;
-	int argi;
+	int argi, i;
 	Source *s;
 	struct block *l;
 	int restricted, errexit;
-	const char **wp;
+	const char **wp, *cc;
 	struct env env;
 	pid_t ppid;
 	struct tbl *vp;
@@ -76,9 +77,6 @@ main(int argc, const char *argv[])
 	size_t k;
 	char *cp;
 #endif
-#if HAVE_SETLOCALE_CTYPE
-	const char *cc;
-#endif
 
 	/* make sure argv[] is sane */
 	if (!*argv) {
@@ -127,10 +125,8 @@ main(int argc, const char *argv[])
 
 	/* define built-in commands */
 	ktinit(&builtins, APERM, 64); /* must be 2^n (currently 40 builtins) */
-	for (i = 0; shbuiltins[i].name != NULL; i++)
-		builtin(shbuiltins[i].name, shbuiltins[i].func);
-	for (i = 0; kshbuiltins[i].name != NULL; i++)
-		builtin(kshbuiltins[i].name, kshbuiltins[i].func);
+	for (i = 0; mkshbuiltins[i].name != NULL; i++)
+		builtin(mkshbuiltins[i].name, mkshbuiltins[i].func);
 
 	init_histvec();
 
@@ -161,6 +157,15 @@ main(int argc, const char *argv[])
 	/* setstr can't fail here */
 	setstr(vp, def_path, KSH_RETURN_ERROR);
 
+	/* Set FPOSIX if we're called as -sh or /bin/sh or so */
+	cc = kshname;
+	i = 0; argi = 0;
+	while (cc[i] != '\0')
+		if ((cc[i++] | 2) == '/')
+			argi = i;
+	if (((cc[argi] | 0x20) == 's') && ((cc[argi + 1] | 0x20) == 'h'))
+		Flag(FPOSIX) = 1;
+
 	/* Turn on nohup by default for now - will change to off
 	 * by default once people are aware of its existence
 	 * (at&t ksh does not have a nohup option - it always sends
@@ -177,7 +182,9 @@ main(int argc, const char *argv[])
 	 * by the environment or the user.  Also, we want tab completion
 	 * on in vi by default. */
 	change_flag(FEMACS, OF_SPECIAL, 1);
+#ifndef MKSH_NOVI
 	Flag(FVITABCOMPLETE) = 1;
+#endif
 
 	/* import environment */
 	if (environ != NULL)
@@ -214,16 +221,17 @@ main(int argc, const char *argv[])
 			setstr(pwd_v, current_wd, KSH_RETURN_ERROR);
 	}
 	ppid = getppid();
-#if !HAVE_ARC4RANDOM
-	srand(((long)kshname) ^ ((long)time(NULL) * kshpid * ppid));
+	change_random(((unsigned long)kshname) ^
+	    ((unsigned long)time(NULL) * kshpid * ppid));
+#if HAVE_ARC4RANDOM
+	Flag(FARC4RANDOM) = 2;	/* use arc4random(3) until $RANDOM is written */
 #endif
 	setint(global("PPID"), (long)ppid);
 
-	/* execute initialisation statements */
 	for (wp = initcoms; *wp != NULL; wp++) {
 		shcomexec(wp);
-		for (; *wp != NULL; wp++)
-			;
+		while (*wp != NULL)
+			wp++;
 	}
 
 	safe_prompt = (ksheuid = geteuid()) ? "$ " : "# ";
@@ -248,6 +256,13 @@ main(int argc, const char *argv[])
 	if (argi < 0)
 		exit(1);
 
+	if (!Flag(FPOSIX))
+		for (wp = initcoms_korn; *wp != NULL; wp++) {
+			shcomexec(wp);
+			while (*wp != NULL)
+				wp++;
+		}
+
 	if (Flag(FCOMMAND)) {
 		s = pushs(SSTRING, ATEMP);
 		if (!(s->start = s->str = argv[argi++]))
@@ -470,7 +485,7 @@ shell(Source * volatile s, volatile int toplevel)
 		case LSHELL:
 			if (interactive) {
 				if (i == LINTR)
-					shellf(newline);
+					shellf("\n");
 				/* Reset any eof that was read as part of a
 				 * multiline command.
 				 */
@@ -699,11 +714,9 @@ reclaim(void)
 static void
 remove_temps(struct temp *tp)
 {
-
 	for (; tp != NULL; tp = tp->next)
-		if (tp->pid == procpid) {
+		if (tp->pid == procpid)
 			unlink(tp->name);
-		}
 }
 
 /* Initialise tty_fd.  Used for saving/reseting tty modes upon
@@ -721,6 +734,11 @@ tty_init(int init_ttystate)
 	}
 	tty_devtty = 1;
 
+#ifdef _UWIN
+	if (isatty(3))
+		tfd = 3;
+	else
+#endif
 	if ((tfd = open("/dev/tty", O_RDWR, 0)) < 0) {
 		tty_devtty = 0;
 		warningf(false, "No controlling tty (open /dev/tty: %s)",
@@ -936,23 +954,18 @@ ksh_dup2(int ofd, int nfd, int errok)
  * move fd from user space (0<=fd<10) to shell space (fd>=10),
  * set close-on-exec flag.
  */
-int
+short
 savefd(int fd)
 {
-	int nfd;
+	int nfd = fd;
 
-	if (fd < FDBASE) {
-		nfd = fcntl(fd, F_DUPFD, FDBASE);
-		if (nfd < 0) {
-			if (errno == EBADF)
-				return -1;
-			else
-				errorf("too many files open in shell");
-		}
-	} else
-		nfd = fd;
+	if (fd < FDBASE && (nfd = fcntl(fd, F_DUPFD, FDBASE)) < 0 &&
+	    errno == EBADF)
+		return -1;
+	if (nfd < 0 || nfd > SHRT_MAX)
+		errorf("too many files open in shell");
 	fcntl(nfd, F_SETFD, FD_CLOEXEC);
-	return nfd;
+	return ((short)nfd);
 }
 
 void
@@ -999,7 +1012,6 @@ check_fd(const char *name, int mode, const char **emsgp)
 	int fd, fl;
 
 	if (ksh_isdigit(name[0]) && !name[1]) {
-		fd = name[0] - '0';
 		if ((fl = fcntl(fd = name[0] - '0', F_GETFL, 0)) < 0) {
 			if (emsgp)
 				*emsgp = "bad file descriptor";
@@ -1255,12 +1267,6 @@ ktenter(struct table *tp, const char *n, unsigned int h)
 	return p;
 }
 
-void
-ktdelete(struct tbl *p)
-{
-	p->flag = 0;
-}
-
 void
 ktwalk(struct tstate *ts, struct table *tp)
 {
