@@ -1,4 +1,4 @@
-/*	$OpenBSD: misc.c,v 1.30 2006/03/12 00:26:58 deraadt Exp $	*/
+/*	$OpenBSD: misc.c,v 1.32 2007/08/02 11:05:54 fgsch Exp $	*/
 /*	$OpenBSD: path.c,v 1.12 2005/03/30 17:16:37 deraadt Exp $	*/
 
 #include "sh.h"
@@ -6,7 +6,7 @@
 #include <grp.h>
 #endif
 
-__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.52.2.1 2007/05/13 19:29:38 tg Exp $\t"
+__RCSID("$MirOS: src/bin/mksh/misc.c,v 1.67 2007/10/25 15:19:16 tg Exp $\t"
 	MKSH_SH_H_ID);
 
 #undef USE_CHVT
@@ -65,7 +65,7 @@ initctypes(void)
 }
 
 /* Allocate a string of size n+1 and copy upto n characters from the possibly
- * null terminated string s into it.  Always returns a null terminated string
+ * NUL terminated string s into it.  Always returns a NUL terminated string
  * (unless n < 0).
  */
 char *
@@ -98,12 +98,15 @@ Xcheck_grow_(XString *xsp, const char *xp, unsigned more)
 	return xsp->beg + (xp - old_beg);
 }
 
-const struct option options[] = {
+const struct shoption options[] = {
 	/* Special cases (see parse_args()): -A, -o, -s.
 	 * Options are sorted by their longnames - the order of these
 	 * entries MUST match the order of sh_flag F* enumerations in sh.h.
 	 */
 	{ "allexport",	'a',		OF_ANY },
+#if HAVE_ARC4RANDOM
+	{ "arc4random",	  0,		OF_ANY },
+#endif
 	{ "braceexpand",  0,		OF_ANY }, /* non-standard */
 	{ "bgnice",	  0,		OF_ANY },
 	{ NULL,		'c',	    OF_CMDLINE },
@@ -131,10 +134,12 @@ const struct option options[] = {
 	{ "trackall",	'h',		OF_ANY },
 	{ "utf8-hack",	'U',		OF_ANY }, /* non-standard */
 	{ "verbose",	'v',		OF_ANY },
+#ifndef MKSH_NOVI
 	{ "vi",		  0,		OF_ANY },
 	{ "viraw",	  0,		OF_ANY }, /* no effect */
 	{ "vi-tabcomplete",  0,		OF_ANY }, /* non-standard */
 	{ "vi-esccomplete",  0,		OF_ANY }, /* non-standard */
+#endif
 	{ "xtrace",	'x',		OF_ANY },
 	/* Anonymous flags: used internally by shell only
 	 * (not visible to user)
@@ -204,7 +209,7 @@ printoptions(int verbose)
 		for (i = 0; i < NELEM(options); i++)
 			if (Flag(i) && options[i].name)
 				shprintf(" -o %s", options[i].name);
-		shprintf(newline);
+		shf_putc('\n', shl_stdout);
 	}
 }
 
@@ -218,25 +223,31 @@ getoptions(void)
 	for (i = 0; i < NELEM(options); i++)
 		if (options[i].c && Flag(i))
 			*cp++ = options[i].c;
-	*cp = 0;
-	return str_save(m, ATEMP);
+	return (str_nsave(m, cp - m, ATEMP));
 }
 
 /* change a Flag(*) value; takes care of special actions */
 void
 change_flag(enum sh_flag f,
     int what,		/* flag to change */
-    int newval)		/* what is changing the flag (command line vs set) */
+    char newval)	/* what is changing the flag (command line vs set) */
 {
-	int oldval;
+	char oldval;
 
 	oldval = Flag(f);
-	Flag(f) = newval;
+	Flag(f) = newval ? 1 : 0;	/* needed for tristates */
 	if (f == FMONITOR) {
 		if (what != OF_CMDLINE && newval != oldval)
 			j_change();
-	} else if ((f == FVI || f == FEMACS || f == FGMACS) && newval) {
-		Flag(FVI) = Flag(FEMACS) = Flag(FGMACS) = 0;
+	} else if ((
+#ifndef MKSH_NOVI
+	    f == FVI ||
+#endif
+	    f == FEMACS || f == FGMACS) && newval) {
+#ifndef MKSH_NOVI
+		Flag(FVI) =
+#endif
+		    Flag(FEMACS) = Flag(FGMACS) = 0;
 		Flag(f) = newval;
 	} else if (f == FPRIVILEGED && oldval && !newval) {
 		/* Turning off -p? */
@@ -274,11 +285,11 @@ parse_args(const char **argv,
 {
 	static char cmd_opts[NELEM(options) + 5]; /* o:T:\0 */
 	static char set_opts[NELEM(options) + 6]; /* A:o;s\0 */
-	char *opts;
+	char set, *opts;
 	const char *array = NULL;
 	Getopt go;
 	size_t i;
-	int optc, set, sortargs = 0, arrayset = 0;
+	int optc, sortargs = 0, arrayset = 0;
 
 	/* First call?  Build option strings... */
 	if (cmd_opts[0] == '\0') {
@@ -374,8 +385,8 @@ parse_args(const char **argv,
 #else
 			change_flag(FTALKING, OF_CMDLINE, 1);
 			chvt(go.optarg);
-#endif
 			break;
+#endif
 #endif
 
 		case '?':
@@ -392,8 +403,7 @@ parse_args(const char **argv,
 			for (i = 0; i < NELEM(options); i++)
 				if (optc == options[i].c &&
 				    (what & options[i].flags)) {
-					change_flag((enum sh_flag) i, what,
-					    set);
+					change_flag((enum sh_flag)i, what, set);
 					break;
 				}
 			if (i == NELEM(options))
@@ -471,11 +481,11 @@ getn(const char *s, int *ai)
 int
 bi_getn(const char *as, int *ai)
 {
-	int rv = getn(as, ai);
+	int rv;
 
-	if (!rv)
+	if (!(rv = getn(as, ai)))
 		bi_errorf("%s: bad number", as);
-	return rv;
+	return (rv);
 }
 
 /* -------- gmatch.c -------- */
@@ -846,7 +856,7 @@ ksh_getopt(const char **argv, Getopt *go, const char *optionsp)
 			    (go->flags & GF_NONAME) ? "" : argv[0],
 			    (go->flags & GF_NONAME) ? "" : ": ", c);
 			if (go->flags & GF_ERROR)
-				bi_errorf(null);
+				bi_errorfz();
 		}
 		return '?';
 	}
@@ -872,7 +882,7 @@ ksh_getopt(const char **argv, Getopt *go, const char *optionsp)
 			    (go->flags & GF_NONAME) ? "" : argv[0],
 			    (go->flags & GF_NONAME) ? "" : ": ", c);
 			if (go->flags & GF_ERROR)
-				bi_errorf(null);
+				bi_errorfz();
 			return '?';
 		}
 		go->p = 0;
@@ -1425,3 +1435,12 @@ stristr(const char *b, const char *l)
 	return (b - 1);
 }
 #endif
+
+#if !HAVE_EXPSTMT
+bool
+ksh_isspace_(unsigned ksh_isspace_c)
+{
+	return ((ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||
+	    (ksh_isspace_c == 0x20));
+}
+#endif
