@@ -8,39 +8,19 @@
 /*	$OpenBSD: c_test.h,v 1.4 2004/12/20 11:34:26 otto Exp $	*/
 /*	$OpenBSD: tty.h,v 1.5 2004/12/20 11:34:26 otto Exp $	*/
 
-#define MKSH_SH_H_ID "$MirOS: src/bin/mksh/sh.h,v 1.127.2.3 2007/05/26 22:31:21 tg Exp $"
-#define MKSH_VERSION "R29 2007/05/24"
+#define MKSH_SH_H_ID "$MirOS: src/bin/mksh/sh.h,v 1.184 2007/10/25 15:34:57 tg Exp $"
+#define MKSH_VERSION "R32 2007/10/25"
 
 #if HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #include <sys/types.h>
-#if defined(MKSH_INCLUDES_ONLY) || !HAVE_MULTI_IDSTRING
-#undef __RCSID
-#endif
-#if !defined(__RCSID) || !defined(__SCCSID)
-#undef __IDSTRING
-#undef __IDSTRING_CONCAT
-#undef __IDSTRING_EXPAND
-#undef __RCSID
-#undef __SCCSID
-#if HAVE_ATTRIBUTE_USED
-#define __attribute____used__	__attribute__((used))
-#else
-#define __attribute____used__	/* nothing */
-#endif
-#define __IDSTRING_CONCAT(l,p)		__LINTED__ ## l ## _ ## p
-#define __IDSTRING_EXPAND(l,p)		__IDSTRING_CONCAT(l,p)
-#define __IDSTRING(prefix, string)				\
-	static const char __IDSTRING_EXPAND(__LINE__,prefix) []	\
-	    __attribute____used__ = "@(""#)" #prefix ": " string
-#define __RCSID(x)	__IDSTRING(rcsid,x)
-#define __SCCSID(x)	__IDSTRING(sccsid,x)
-#endif
-
 #include <sys/time.h>
 #include <sys/file.h>
 #include <sys/ioctl.h>
+#if HAVE_SYS_SYSMACROS_H
+#include <sys/sysmacros.h>
+#endif
 #if HAVE_SYS_MKDEV_H
 #include <sys/mkdev.h>
 #endif
@@ -49,9 +29,6 @@
 #endif
 #include <sys/resource.h>
 #include <sys/stat.h>
-#if HAVE_SYS_SYSMACROS_H
-#include <sys/sysmacros.h>
-#endif
 #include <sys/wait.h>
 #include <dirent.h>
 #include <errno.h>
@@ -59,6 +36,9 @@
 #if HAVE_LIBGEN_H
 #include <libgen.h>
 #endif
+#if HAVE_LIBUTIL_H
+#include <libutil.h>
+#endif
 #include <limits.h>
 #if HAVE_PATHS_H
 #include <paths.h>
@@ -87,6 +67,37 @@
 #include <values.h>
 #endif
 
+#if HAVE_ATTRIBUTE
+#undef __attribute__
+#else
+#define __attribute__(x)	/* nothing */
+#endif
+#undef __unused
+#define __unused		__attribute__((unused))
+#if HAVE_ATTRIBUTE_BOUNDED
+#define __bound_att__(x)	__attribute__(x)
+#else
+#define __bound_att__(x)	/* nothing */
+#endif
+#if HAVE_ATTRIBUTE_USED
+#define __attribute____used__	__attribute__((used))
+#else
+#define __attribute____used__	/* nothing */
+#endif
+
+#undef __IDSTRING
+#undef __IDSTRING_CONCAT
+#undef __IDSTRING_EXPAND
+#undef __RCSID
+#undef __SCCSID
+#define __IDSTRING_CONCAT(l,p)		__LINTED__ ## l ## _ ## p
+#define __IDSTRING_EXPAND(l,p)		__IDSTRING_CONCAT(l,p)
+#define __IDSTRING(prefix, string)				\
+	static const char __IDSTRING_EXPAND(__LINE__,prefix) []	\
+	    __attribute____used__ = "@(""#)" #prefix ": " string
+#define __RCSID(x)	__IDSTRING(rcsid,x)
+#define __SCCSID(x)	__IDSTRING(sccsid,x)
+
 #ifndef MKSH_INCLUDES_ONLY
 
 /* extra types */
@@ -96,6 +107,7 @@ typedef long rlim_t;
 #endif
 
 #if !HAVE_SIG_T
+#undef sig_t
 typedef void (*sig_t)(int);
 #endif
 
@@ -108,6 +120,9 @@ typedef int bool;
 
 /* extra macros */
 
+#ifndef timerclear
+#define timerclear(tvp) do { (tvp)->tv_sec = (tvp)->tv_usec = 0; } while (0)
+#endif
 #ifndef timeradd
 #define timeradd(tvp, uvp, vvp)						\
 	do {								\
@@ -138,25 +153,16 @@ typedef int bool;
 #define ksh_toupper(c)	(((c) >= 'a') && ((c) <= 'z') ? (c) - 'a' + 'A' : (c))
 #define ksh_isdash(s)	(((s) != NULL) && ((s)[0] == '-') && ((s)[1] == '\0'))
 
-#if HAVE_ATTRIBUTE
-#undef __attribute__
-#if HAVE_ATTRIBUTE_BOUNDED
-#define __bound_att__		__attribute__
-#else
-#define __bound_att__(x)	/* nothing */
-#endif
-#else
-#define __attribute__(x)	/* nothing */
-#endif
-#undef __unused
-#define __unused		__attribute__((unused))
-
+#if HAVE_EXPSTMT
 /* this macro must not evaluate its arguments several times */
-#define ksh_isspace(c)	__extension__({					\
-		unsigned char ksh_isspace_c = (c);			\
-		(ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||	\
-		    (ksh_isspace_c == 0x20);				\
-	})
+#define ksh_isspace(c)	({					\
+	unsigned ksh_isspace_c = (c);				\
+	(ksh_isspace_c >= 0x09 && ksh_isspace_c <= 0x0D) ||	\
+	    (ksh_isspace_c == 0x20);				\
+})
+#else
+#define ksh_isspace(c)	ksh_isspace_((unsigned)(c))
+#endif
 
 #ifndef S_ISTXT
 #define S_ISTXT		0001000
@@ -186,14 +192,12 @@ typedef int bool;
 
 #if !HAVE_ARC4RANDOM_DECL
 extern u_int32_t arc4random(void);
+extern void arc4random_addrandom(unsigned char *, int)
+    __bound_att__((bounded (string, 1, 2)));
 #endif
 
-#if !HAVE_ARC4RANDOM_PUSH_DECL
-extern void arc4random_push(int);
-#endif
-
-#if !HAVE_CONFSTR_DECL
-size_t confstr(int, char *, size_t);
+#if !HAVE_ARC4RANDOM_PUSHB_DECL
+extern uint32_t arc4random_pushb(void *, size_t);
 #endif
 
 #if !HAVE_SETMODE
@@ -274,58 +278,46 @@ union mksh_ccphack {
 };
 
 /* for const debugging */
-#ifdef DEBUG
+#if defined(DEBUG) && defined(__GNUC__) && !defined(__ICC) && \
+    !defined(__INTEL_COMPILER) && !defined(__SUNPRO_C)
 char *ucstrchr(char *, int);
 char *ucstrstr(char *, const char *);
 #define strchr ucstrchr
 #define strstr ucstrstr
-#define cstrchr(s,c) __extension__({	\
+#define cstrchr(s,c) ({			\
 	union mksh_cchack in, out;	\
 					\
 	in.ro = (s);			\
-	out.rw = strchr(in.rw, (c));	\
+	out.rw = ucstrchr(in.rw, (c));	\
 	(out.ro);			\
 })
-#define cstrstr(b,l) __extension__({	\
+#define cstrstr(b,l) ({			\
 	union mksh_cchack in, out;	\
 					\
 	in.ro = (b);			\
-	out.rw = strstr(in.rw, (l));	\
+	out.rw = ucstrstr(in.rw, (l));	\
 	(out.ro);			\
 })
 #define vstrchr(s,c)	(cstrchr((s), (c)) != NULL)
 #define vstrstr(b,l)	(cstrstr((b), (l)) != NULL)
-#if HAVE_STRCASESTR
-#define stristr(b,l) __extension__({	\
-	union mksh_cchack out;		\
-					\
-	out.rw = strcasestr((b), (l));	\
-	(out.ro);			\
-})
-#endif
-#else
-#define cstrchr(s,c) __extension__({	\
-	union mksh_cchack out;		\
-					\
-	out.rw = strchr((s), (c));	\
-	(out.ro);			\
-})
-#define cstrstr(b,l) __extension__({	\
-	union mksh_cchack out;		\
-					\
-	out.rw = strstr((b), (l));	\
-	(out.ro);			\
-})
-#define vstrchr	strchr
-#define vstrstr	strstr
-#if HAVE_STRCASESTR
-#define stristr	strcasestr
-#endif
+#else /* !DEBUG, !gcc */
+#define cstrchr(s,c)	((const char *)strchr((s), (c)))
+#define cstrstr(s,c)	((const char *)strstr((s), (c)))
+#define vstrchr(s,c)	(strchr((s), (c)) != NULL)
+#define vstrstr(b,l)	(strstr((b), (l)) != NULL)
 #endif
 
 /* use this ipv strchr(s, 0) but no side effects in s! */
 #define strnul(s)	((s) + strlen(s))
 
+#if HAVE_STRCASESTR
+#define stristr(b,l)	((const char *)strcasestr((b), (l)))
+#endif
+
+#if defined(MKSH_SMALL) && !defined(MKSH_NOVI)
+#define MKSH_NOVI
+#endif
+
 /*
  * Area-based allocation built on malloc/free
  */
@@ -341,14 +333,14 @@ EXTERN Area aperm;		/* permanent object space */
  * parsing & execution environment
  */
 EXTERN struct env {
-	short type;		/* environment type - see below */
-	short flags;		/* EF_* */
 	Area area;		/* temporary allocation area */
 	struct block *loc;	/* local variables and functions */
 	short *savefd;		/* original redirected fds */
 	struct env *oenv;	/* link to previous environment */
-	sigjmp_buf jbuf;	/* long jump back to env creator */
 	struct temp *temps;	/* temp files */
+	sigjmp_buf jbuf;	/* long jump back to env creator */
+	short type;		/* environment type - see below */
+	short flags;		/* EF_* */
 } *e;
 
 /* struct env.type values */
@@ -391,18 +383,21 @@ EXTERN struct env {
 #define OF_FIRSTTIME	0x10	/* as early as possible, once */
 #define OF_ANY		(OF_CMDLINE | OF_SET | OF_SPECIAL | OF_INTERNAL)
 
-struct option {
+struct shoption {
 	const char *name;	/* long name of option */
 	char c;			/* character flag (if any) */
 	unsigned char flags;	/* OF_* */
 };
-extern const struct option options[];
+extern const struct shoption options[];
 
 /*
  * flags (the order of these enums MUST match the order in misc.c(options[]))
  */
 enum sh_flag {
 	FEXPORT = 0,	/* -a: export all */
+#if HAVE_ARC4RANDOM
+	FARC4RANDOM,	/* use 0:rand(3) 1:arc4random(3) 2:switch on write */
+#endif
 	FBRACEEXPAND,	/* enable {} globbing */
 	FBGNICE,	/* bgnice */
 	FCOMMAND,	/* -c: (invocation) execute specified command */
@@ -430,22 +425,25 @@ enum sh_flag {
 	FTRACKALL,	/* -h: create tracked aliases for all commands */
 	FUTFHACK,	/* -U: utf-8 hack for command line editing */
 	FVERBOSE,	/* -v: echo input */
+#ifndef MKSH_NOVI
 	FVI,		/* vi command editing */
 	FVIRAW,		/* always read in raw mode (ignored) */
 	FVITABCOMPLETE,	/* enable tab as file name completion char */
 	FVIESCCOMPLETE,	/* enable ESC as file name completion in command mode */
+#endif
 	FXTRACE,	/* -x: execution trace */
 	FTALKING_I,	/* (internal): initial shell was interactive */
 	FNFLAGS		/* (place holder: how many flags are there) */
 };
 
-#define Flag(f)	(shell_flags[(int) (f)])
+#define Flag(f)	(shell_flags[(int)(f)])
 
-EXTERN char shell_flags [FNFLAGS];
+EXTERN char shell_flags[FNFLAGS];
 
-EXTERN char	null [] I__("");	/* null value for variable */
-EXTERN char	space [] I__(" ");
-EXTERN char	newline [] I__("\n");
+/* null value for variable; comparision pointer for unset */
+EXTERN char null[] I__("");
+/* helpers for string pooling */
+EXTERN const char T_synerr[] I__("syntax error");
 
 enum temp_type {
 	TT_HEREDOC_EXP,	/* expanded heredoc */
@@ -688,8 +686,8 @@ extern struct shf shf_iob[];
 
 struct table {
 	Area *areap;		/* area to allocate entries */
-	short size, nfree;	/* hash size (always 2^^n), free entries */
 	struct tbl **tbls;	/* hashed table items */
+	short size, nfree;	/* hash size (always 2^^n), free entries */
 };
 
 struct tbl {			/* table item */
@@ -703,7 +701,7 @@ struct tbl {			/* table item */
 		int (*f)(const char **);/* int function */
 		struct op *t;		/* "function" tree */
 	} val;			/* value */
-	int index;		/* index for an array */
+	uint32_t index;		/* index for an array */
 	union {
 		int field;	/* field with for -L/-R/-Z */
 		int errno_;	/* CEXEC/CTALIAS */
@@ -824,7 +822,7 @@ struct builtin {
 	int (*func)(const char **);
 };
 
-extern const struct builtin shbuiltins [], kshbuiltins [];
+extern const struct builtin mkshbuiltins[];
 
 /* var spec values */
 #define V_NONE		0
@@ -859,11 +857,6 @@ EXTERN int current_lineno;	/* LINENO value */
  * Description of a command or an operation on commands.
  */
 struct op {
-	short type;			/* operation type, see below */
-	union { /* WARNING: newtp(), tcopy() use evalflags = 0 to clear union */
-		short evalflags;	/* TCOM: arg expansion eval() flags */
-		short ksh_func;		/* TFUNC: function x (vs x()) */
-	} u;
 	const char **args;		/* arguments to a command */
 	char **vars;			/* variable assignments */
 	struct ioword **ioact;		/* IO actions (eg, < > >>) */
@@ -874,6 +867,11 @@ struct op {
 					 * time hook for TCOM.
 					 */
 	int lineno;			/* TCOM/TFUNC: LINENO for this */
+	short type;			/* operation type, see below */
+	union { /* WARNING: newtp(), tcopy() use evalflags = 0 to clear union */
+		short evalflags;	/* TCOM: arg expansion eval() flags */
+		short ksh_func;		/* TFUNC: function x (vs x()) */
+	} u;
 };
 
 /* Tree.type values */
@@ -916,6 +914,7 @@ struct op {
 #define OPAT	9	/* open pattern: *(, @(, etc. */
 #define SPAT	10	/* separate pattern: | */
 #define CPAT	11	/* close pattern: ) */
+#define ADELIM	12	/* arbitrary delimiter: ${foo:2:3} ${foo/bar/baz} */
 
 /*
  * IO redirection
@@ -1127,6 +1126,8 @@ struct source {
 #define SHEREDQUOTE	10	/* parsing " in <<,<<- delimiter */
 #define SPATTERN	11	/* parsing *(...|...) pattern (*+?@!) */
 #define STBRACE		12	/* parsing ${..[#%]..} */
+#define SLETARRAY	13	/* inside =( ), just copy */
+#define SADELIM		14	/* like SBASE, looking for delimiter */
 
 typedef union {
 	int i;
@@ -1177,6 +1178,7 @@ typedef union {
 #define HEREDELIM	BIT(9)	/* parsing <<,<<- delimiter */
 #define LQCHAR		BIT(10)	/* source string contains QCHAR */
 #define HEREDOC		BIT(11)	/* parsing a here document */
+#define LETARRAY	BIT(12)	/* copy expression inside =( ) */
 
 #define HERES	10		/* max << in line */
 
@@ -1273,6 +1275,10 @@ void timex_hook(struct op *, char ** volatile *);
 int c_exec(const char **);
 int c_builtin(const char **);
 int c_test(const char **);
+#if HAVE_MKNOD
+int c_mknod(const char **);
+#endif
+int c_rename(const char **);
 /* histrap.c */
 void init_histvec(void);
 void hist_init(Source *);
@@ -1347,6 +1353,18 @@ void warningf(bool, const char *, ...)
     __attribute__((format (printf, 2, 3)));
 void bi_errorf(const char *, ...)
     __attribute__((format (printf, 1, 2)));
+/*
+ * circumvent compiler format string nonnull checking
+ * we teach xlC to not bitch about zero-lengths, want
+ * gcc to do it, and so gain double-checking benefits
+ */
+#if defined(__xlC__)
+#define errorfz()	errorf("")
+#define bi_errorfz()	bi_errorf("")
+#else
+#define errorfz()	errorf(null)
+#define bi_errorfz()	bi_errorf(null)
+#endif
 void internal_errorf(const char *, ...)
     __attribute__((noreturn))
     __attribute__((format (printf, 1, 2)));
@@ -1360,7 +1378,7 @@ void shprintf(const char *, ...)
 int can_seek(int);
 void initio(void);
 int ksh_dup2(int, int, int);
-int savefd(int);
+short savefd(int);
 void restfd(int, int);
 void openpipe(int *);
 void closepipe(int *);
@@ -1376,7 +1394,7 @@ unsigned int hash(const char *);
 void ktinit(struct table *, Area *, int);
 struct tbl *ktsearch(struct table *, const char *, unsigned int);
 struct tbl *ktenter(struct table *, const char *, unsigned int);
-void ktdelete(struct tbl *);
+#define ktdelete(p)	do { p->flag = 0; } while (0)
 void ktwalk(struct tstate *, struct table *);
 struct tbl *ktnext(struct tstate *);
 struct tbl **ktsort(struct table *);
@@ -1391,7 +1409,7 @@ char *str_save(const char *, Area *);
 char *str_nsave(const char *, int, Area *);
 size_t option(const char *);
 char *getoptions(void);
-void change_flag(enum sh_flag, int, int);
+void change_flag(enum sh_flag, int, char);
 int parse_args(const char **, int, int *);
 int getn(const char *, int *);
 int bi_getn(const char *, int *);
@@ -1406,13 +1424,17 @@ void print_columns(struct shf *, int,
     char *(*)(const void *, int, char *, int),
     const void *, int, int prefcol);
 void strip_nuls(char *, int);
-int blocking_read(int, char *, int);
+int blocking_read(int, char *, int)
+    __bound_att__((bounded (buffer, 2, 3)));
 int reset_nonblock(int);
 char *ksh_get_wd(size_t *);
 int make_path(const char *, const char *, char **, XString *, int *);
 void simplify_path(char *);
 char *get_phys_path(const char *);
 void set_current_wd(char *);
+#if !HAVE_EXPSTMT
+bool ksh_isspace_(unsigned);
+#endif
 /* shf.c */
 struct shf *shf_open(const char *, int, int, int);
 struct shf *shf_fdopen(int, int, struct shf *);
@@ -1468,7 +1490,7 @@ const char *skip_wdvarname(const char *, int);
 int is_wdvarname(const char *, int);
 int is_wdvarassign(const char *);
 char **makenv(void);
-void change_random(void);
+void change_random(unsigned long);
 int array_ref_len(const char *);
 char *arrayname(const char *);
 void set_array(const char *, int, const char **);
@@ -1518,6 +1540,8 @@ struct test_env {
 	void (*error)(Test_env *, int, const char *);
 };
 
+extern const char *const dbtest_tokens[];
+
 Test_op	test_isop(Test_meta, const char *);
 int test_eval(Test_env *, Test_op, const char *, const char *, int);
 int test_parse(Test_env *);
