@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/shf.c,v 1.13.2.1 2007/05/13 19:29:39 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/shf.c,v 1.16 2007/10/25 15:23:10 tg Exp $");
 
 /* flags to shf_emptybuf() */
 #define EB_READSW	0x01	/* about to switch to reading */
@@ -553,7 +553,7 @@ shf_putchar(int c, struct shf *shf)
 		return EOF;
 
 	if (shf->flags & SHF_UNBUF) {
-		char cc = c;
+		unsigned char cc = (unsigned char)c;
 		int n;
 
 		if (shf->fd < 0)
@@ -924,7 +924,7 @@ shf_vfprintf(struct shf *shf, const char *fmt, va_list args)
 
 		case 'c':
 			flags &= ~FL_DOT;
-			numbuf[0] = va_arg(args, int);
+			numbuf[0] = (char)(va_arg(args, int));
 			s = numbuf;
 			len = 1;
 			break;
