@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.12.2.1 2007/05/13 19:29:40 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/syn.c,v 1.18 2007/10/25 15:34:30 tg Exp $");
 
 struct nesting_state {
 	int start_token;	/* token than began nesting (eg, FOR) */
@@ -14,7 +14,6 @@ static struct op *pipeline(int);
 static struct op *andor(void);
 static struct op *c_list(int);
 static struct ioword *synio(int);
-static void musthave(int, int);
 static struct op *nested(int, int, int);
 static struct op *get_command(int);
 static struct op *dogroup(void);
@@ -49,6 +48,7 @@ static int symbol;		/* yylex value */
 #define	ACCEPT		(reject = 0)
 #define	token(cf)	((reject) ? (ACCEPT, symbol) : (symbol = yylex(cf)))
 #define	tpeek(cf)	((reject) ? (symbol) : (REJECT, symbol = yylex(cf)))
+#define musthave(c,cf)	do { if (token(cf) != (c)) syntaxerr(NULL); } while (0)
 
 static void
 yyparse(void)
@@ -164,13 +164,6 @@ synio(int cf)
 	return iop;
 }
 
-static void
-musthave(int c, int cf)
-{
-	if ((token(cf)) != c)
-		syntaxerr(NULL);
-}
-
 static struct op *
 nested(int type, int smark, int emark)
 {
@@ -248,6 +241,9 @@ get_command(int cf)
 					ACCEPT;
 					goto Subshell;
 				}
+				if ((XPsize(args) == 0 || Flag(FKEYWORD)) &&
+				    XPsize(vars) == 1 && is_wdvarassign(yylval.cp))
+					goto is_wdarrassign;
 				/* Must be a function */
 				if (iopn != 0 || XPsize(args) != 1 ||
 				    XPsize(vars) != 0)
@@ -257,6 +253,41 @@ get_command(int cf)
 				musthave(')', 0);
 				t = function_body(XPptrv(args)[0], false);
 				goto Leave;
+ is_wdarrassign:
+			  {
+				static const char set_cmd0[] = {
+					CHAR, 'e', CHAR, 'v',
+					CHAR, 'a', CHAR, 'l', EOS
+				};
+				static const char set_cmd1[] = {
+					CHAR, 's', CHAR, 'e',
+					CHAR, 't', CHAR, ' ',
+					CHAR, '-', CHAR, 'A', EOS
+				};
+				static const char set_cmd2[] = {
+					CHAR, '-', CHAR, '-', EOS
+				};
+				char *tcp;
+				XPfree(vars);
+				XPinit(vars, 16);
+				/*
+				 * we know (or rather hope) that yylval.cp
+				 * contains a string "varname="
+				 */
+				tcp = wdcopy(yylval.cp, ATEMP);
+				tcp[wdscan(tcp, EOS) - tcp - 3] = EOS;
+				/* now make an array assignment command */
+				t = newtp(TCOM);
+				t->lineno = source->line;
+				ACCEPT;
+				XPput(args, wdcopy(set_cmd0, ATEMP));
+				XPput(args, wdcopy(set_cmd1, ATEMP));
+				XPput(args, tcp);
+				XPput(args, wdcopy(set_cmd2, ATEMP));
+				musthave(LWORD,LETARRAY);
+				XPput(args, yylval.cp);
+				break;
+			  }
 
 			default:
 				goto Leave;
@@ -315,7 +346,7 @@ get_command(int cf)
 		if (!is_wdvarname(yylval.cp, true))
 			yyerror("%s: bad identifier\n",
 			    c == FOR ? "for" : "select");
-		t->str = str_save(ident, ATEMP);
+		t->str = str_nsave(ident, strlen(ident), ATEMP);
 		nesting_push(&old_nesting, c);
 		t->vars = wordlist();
 		t->left = dogroup();
@@ -691,7 +722,7 @@ syntaxerr(const char *what)
 			goto Again;
 		}
 		/* don't quote the EOF */
-		yyerror("syntax error: unexpected EOF\n");
+		yyerror("%s: unexpected EOF\n", T_synerr);
 		/* NOTREACHED */
 
 	case LWORD:
@@ -718,7 +749,7 @@ syntaxerr(const char *what)
 			s = redir;
 		}
 	}
-	yyerror("syntax error: '%s' %s\n", s, what);
+	yyerror("%s: '%s' %s\n", T_synerr, s, what);
 }
 
 static void
@@ -820,7 +851,7 @@ static int
 dbtestp_isa(Test_env *te, Test_meta meta)
 {
 	int c = tpeek(ARRAYVAR | (meta == TM_BINOP ? 0 : CONTIN));
-	int uqword = 0;
+	int uqword;
 	char *save = NULL;
 	int ret = 0;
 
