@@ -2,7 +2,7 @@
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.9.2.1 2007/05/13 19:29:40 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/tree.c,v 1.12 2007/10/25 15:19:16 tg Exp $");
 
 #define INDENT	4
 
@@ -268,6 +268,7 @@ tputS(char *wp, struct shf *shf)
 		switch ((c = *wp++)) {
 		case EOS:
 			return;
+		case ADELIM:
 		case CHAR:
 			tputC(*wp++, shf);
 			break;
@@ -335,29 +336,29 @@ tputS(char *wp, struct shf *shf)
 int
 fptreef(struct shf *shf, int indent, const char *fmt, ...)
 {
-  va_list	va;
+	va_list va;
 
-  va_start(va, fmt);
+	va_start(va, fmt);
 
-  vfptreef(shf, indent, fmt, va);
-  va_end(va);
-  return 0;
+	vfptreef(shf, indent, fmt, va);
+	va_end(va);
+	return (0);
 }
 
 /* VARARGS */
 char *
 snptreef(char *s, int n, const char *fmt, ...)
 {
-  va_list va;
-  struct shf shf;
+	va_list va;
+	struct shf shf;
 
-  shf_sopen(s, n, SHF_WR | (s ? 0 : SHF_DYNAMIC), &shf);
+	shf_sopen(s, n, SHF_WR | (s ? 0 : SHF_DYNAMIC), &shf);
 
-  va_start(va, fmt);
-  vfptreef(&shf, 0, fmt, va);
-  va_end(va);
+	va_start(va, fmt);
+	vfptreef(&shf, 0, fmt, va);
+	va_end(va);
 
-  return shf_sclose(&shf); /* null terminates */
+	return (shf_sclose(&shf)); /* null terminates */
 }
 
 static void
@@ -484,6 +485,10 @@ wdscan(const char *wp, int c)
 		switch (*wp++) {
 		case EOS:
 			return (wp);
+		case ADELIM:
+			if (c == ADELIM)
+				return (wp + 1);
+			/* FALLTHROUGH */
 		case CHAR:
 		case QCHAR:
 			wp++;
@@ -546,6 +551,7 @@ wdstrip(const char *wp)
 		switch ((c = *wp++)) {
 		case EOS:
 			return shf_sclose(&shf); /* null terminates */
+		case ADELIM:
 		case CHAR:
 		case QCHAR:
 			shf_putchar(*wp++, &shf);
