@@ -1,8 +1,8 @@
-/*	$OpenBSD: var.c,v 1.30 2006/05/21 18:40:39 otto Exp $	*/
+/*	$OpenBSD: var.c,v 1.33 2007/08/02 11:05:54 fgsch Exp $	*/
 
 #include "sh.h"
 
-__RCSID("$MirOS: src/bin/mksh/var.c,v 1.37.2.1 2007/05/13 19:29:40 tg Exp $");
+__RCSID("$MirOS: src/bin/mksh/var.c,v 1.50 2007/10/25 16:10:16 tg Exp $");
 
 /*
  * Variables
@@ -22,7 +22,10 @@ static void	unspecial(const char *);
 static void	getspec(struct tbl *);
 static void	setspec(struct tbl *);
 static void	unsetspec(struct tbl *);
-static struct tbl *arraysearch(struct tbl *, int);
+static struct tbl *arraysearch(struct tbl *, uint32_t);
+static const char *array_index_calc(const char *, bool *, uint32_t *);
+static int rnd_get(void);
+static void rnd_set(unsigned long);
 
 /*
  * create a new block for function calls and simple commands
@@ -114,7 +117,7 @@ initvar(void)
  * the basename of the array.
  */
 static const char *
-array_index_calc(const char *n, bool *arrayp, int *valp)
+array_index_calc(const char *n, bool *arrayp, uint32_t *valp)
 {
 	const char *p;
 	int len;
@@ -132,9 +135,7 @@ array_index_calc(const char *n, bool *arrayp, int *valp)
 		afree(tmp, ATEMP);
 		n = str_nsave(n, p - n, ATEMP);
 		evaluate(sub, &rval, KSH_UNWIND_ERROR, true);
-		if (rval < 0 || rval > 2147483647)
-			errorf("%s: subscript %ld out of range", n, rval);
-		*valp = rval;
+		*valp = (uint32_t)rval;
 		afree(sub, ATEMP);
 	}
 	return n;
@@ -150,8 +151,8 @@ global(const char *n)
 	struct tbl *vp;
 	int c;
 	unsigned h;
-	bool	 array;
-	int	 val;
+	bool array;
+	uint32_t val;
 
 	/* Check to see if this is an array */
 	n = array_index_calc(n, &array, &val);
@@ -231,8 +232,8 @@ local(const char *n, bool copy)
 	struct block *l = e->loc;
 	struct tbl *vp;
 	unsigned h;
-	bool	 array;
-	int	 val;
+	bool array;
+	uint32_t val;
 
 	/* Check to see if this is an array */
 	n = array_index_calc(n, &array, &val);
@@ -336,12 +337,14 @@ intval(struct tbl *vp)
 int
 setstr(struct tbl *vq, const char *s, int error_ok)
 {
+	char *salloc = NULL;
 	int no_ro_check = error_ok & 0x4;
+
 	error_ok &= ~0x4;
 	if ((vq->flag & RDONLY) && !no_ro_check) {
 		warningf(true, "%s: is read only", vq->name);
 		if (!error_ok)
-			errorf(null);
+			errorfz();
 		return 0;
 	}
 	if (!(vq->flag&INTEGER)) { /* string dest */
@@ -357,7 +360,7 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 		vq->flag &= ~(ISSET|ALLOC);
 		vq->type = 0;
 		if (s && (vq->flag & (UCASEV_AL|LCASEV|LJUST|RJUST)))
-			s = formatstr(vq, s);
+			s = salloc = formatstr(vq, s);
 		if ((vq->flag&EXPORT))
 			export(vq, s);
 		else {
@@ -366,12 +369,13 @@ setstr(struct tbl *vq, const char *s, int error_ok)
 		}
 	} else {		/* integer dest */
 		if (!v_evaluate(vq, s, error_ok, true))
-			return 0;
+			return (0);
 	}
 	vq->flag |= ISSET;
 	if ((vq->flag&SPECIAL))
 		setspec(vq);
-	return 1;
+	afreechk(salloc);
+	return (1);
 }
 
 /* set variable to integer */
@@ -690,7 +694,7 @@ typeset(const char *var, Tflag set, Tflag clr, int field, int base)
 			}
 		}
 		if (!ok)
-		    errorf(null);
+			errorfz();
 	}
 
 	if (val != NULL) {
@@ -814,7 +818,7 @@ is_wdvarassign(const char *s)
 char **
 makenv(void)
 {
-	struct block *l = e->loc;
+	struct block *l;
 	XPtrV env;
 	struct tbl *vp, **vpp;
 	int i;
@@ -849,25 +853,95 @@ makenv(void)
 }
 
 /*
- * Someone has set the srand() value, therefore from now on
- * we return values from rand() instead of arc4random()
+ * Get us a random number, either from rand(3) or arc4random(3), with
+ * the latter being preferred. If Flag(FARC4RANDOM) is 0, we use rand(3),
+ * otherwise arc4random(3). We have static caches to make change_random
+ * and writes to $RANDOM a cheap operation.
  */
 #if HAVE_ARC4RANDOM
-static int use_rand = 0;
+static uint32_t rnd_cache[2];
+static char rnd_lastflag = 2;
 #endif
 
+static int
+rnd_get(void)
+{
+#if HAVE_ARC4RANDOM
+#if HAVE_ARC4RANDOM_PUSHB
+	uint32_t rv = 0;
+#endif
+	if (Flag(FARC4RANDOM) != rnd_lastflag) {
+		if (Flag(FARC4RANDOM) == 0) {
+			/* transition to 0 by set: srand */
+			srand(arc4random() & 0x7FFF);
+		} else if (rnd_lastflag == 0) {
+			/* transition from 0: addrandom */
+			rnd_cache[0] ^= rand();
+			rnd_cache[1] ^= rand();
+		}
+		rnd_lastflag = Flag(FARC4RANDOM);
+	}
+	if (Flag(FARC4RANDOM)) {
+		if (rnd_cache[0] || rnd_cache[1])
+#if HAVE_ARC4RANDOM_PUSHB
+			rv = arc4random_pushb(rnd_cache, sizeof (rnd_cache));
+#else
+			arc4random_addrandom((void *)rnd_cache,
+			    sizeof (rnd_cache));
+#endif
+		rnd_cache[0] = rnd_cache[1] = 0;
+		return ((
+#if HAVE_ARC4RANDOM_PUSHB
+		    rv ? rv :
+#endif
+		    arc4random()) & 0x7FFF);
+	}
+#endif
+	return (rand() & 0x7FFF);
+}
+
+static void
+rnd_set(unsigned long newval)
+{
+#if HAVE_ARC4RANDOM
+	rnd_cache[0] ^= (newval << 15) | rand();
+	rnd_cache[1] ^= newval >> 17;
+	if (Flag(FARC4RANDOM) == 1)
+		return;
+	if (Flag(FARC4RANDOM) == 2)
+		Flag(FARC4RANDOM) = 0;
+	/* transition to 0 by write: only srand */
+	rnd_lastflag = 0;
+#endif
+	srand(newval & 0x7FFF);
+}
+
 /*
  * Called after a fork in parent to bump the random number generator.
  * Done to ensure children will not get the same random number sequence
  * if the parent doesn't use $RANDOM.
  */
 void
-change_random(void)
+change_random(unsigned long newval)
 {
+	int rval = 0;
+
 #if HAVE_ARC4RANDOM
-	if (use_rand)
+	if (Flag(FARC4RANDOM)) {
+		rnd_cache[0] ^= (newval << 15) | rand();
+		rnd_cache[1] ^= newval >> 17;
+		return;
+	}
 #endif
-		rand();
+
+	rval += newval & 0x7FFF;
+	newval >>= 15;
+	rval += newval & 0x7FFF;
+	newval >>= 15;
+	rval += newval + rand();
+	rval = (rval & 0x7FFF) ^ (rval >> 15);
+
+	srand(rval);
 }
 
 /*
@@ -918,12 +992,7 @@ getspec(struct tbl *vp)
 		break;
 	case V_RANDOM:
 		vp->flag &= ~SPECIAL;
-#if HAVE_ARC4RANDOM
-		if (!use_rand)
-			setint(vp, arc4random() & 0x7FFF);
-		else
-#endif
-			setint(vp, rand() & 0x7FFF);
+		setint(vp, rnd_get());
 		vp->flag |= SPECIAL;
 		break;
 	case V_HISTSIZE:
@@ -953,7 +1022,8 @@ setspec(struct tbl *vp)
 	case V_PATH:
 		if (path)
 			afree(path, APERM);
-		path = str_save(str_val(vp), APERM);
+		s = str_val(vp);
+		path = str_save(s, APERM);
 		flushcom(1);	/* clear tracked aliases */
 		break;
 	case V_IFS:
@@ -962,7 +1032,7 @@ setspec(struct tbl *vp)
 		break;
 	case V_OPTIND:
 		vp->flag &= ~SPECIAL;
-		getopts_reset((int) intval(vp));
+		getopts_reset((int)intval(vp));
 		vp->flag |= SPECIAL;
 		break;
 	case V_TMPDIR:
@@ -984,7 +1054,7 @@ setspec(struct tbl *vp)
 		break;
 	case V_HISTSIZE:
 		vp->flag &= ~SPECIAL;
-		sethistsize((int) intval(vp));
+		sethistsize((int)intval(vp));
 		vp->flag |= SPECIAL;
 		break;
 #if HAVE_PERSISTENT_HISTORY
@@ -998,13 +1068,7 @@ setspec(struct tbl *vp)
 		break;
 	case V_RANDOM:
 		vp->flag &= ~SPECIAL;
-		srand((unsigned int)intval(vp));
-#if HAVE_ARC4RANDOM
-		use_rand = 1;
-#if HAVE_ARC4RANDOM_PUSH
-		arc4random_push((unsigned)vp ^ (unsigned)rand());
-#endif
-#endif
+		rnd_set(intval(vp));
 		vp->flag |= SPECIAL;
 		break;
 	case V_SECONDS:
@@ -1025,7 +1089,7 @@ setspec(struct tbl *vp)
 	case V_LINENO:
 		vp->flag &= ~SPECIAL;
 		/* The -1 is because line numbering starts at 1. */
-		user_lineno = (unsigned int) intval(vp) - current_lineno - 1;
+		user_lineno = (unsigned int)intval(vp) - current_lineno - 1;
 		vp->flag |= SPECIAL;
 		break;
 	}
@@ -1075,7 +1139,7 @@ unsetspec(struct tbl *vp)
  * vp, indexed by val.
  */
 static struct tbl *
-arraysearch(struct tbl *vp, int val)
+arraysearch(struct tbl *vp, uint32_t val)
 {
 	struct tbl *prev, *curr, *new;
 	size_t namelen = strlen(vp->name) + 1;
@@ -1152,7 +1216,7 @@ void
 set_array(const char *var, int reset, const char **vals)
 {
 	struct tbl *vp, *vq;
-	int i;
+	uint32_t i;
 
 	/* to get local array, use "typeset foo; set -A foo" */
 	vp = global(var);
